package com.showmejava.gridbag;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;


/**
 * Displays the generated source code.  The code in this window will not
 * automatically re-generate every time the constraints are changed.  This
 * approach improves performance as constraints are modified, however
 * the user does have to remember to click the "Generate" button in order
 * to ensure that the generated code is in sync with the model.
 * @author Eric M. Burke
 * @version 2.0
 */
public class CodeFrame extends JFrame implements ActionListener {

    private ConstraintsModel model;
    private CodeGenerator codeGen;
    private JTextArea codeArea = new JTextArea(20,60);
    private JButton generateBtn = new JButton("Generate");
    private JButton copyBtn = new JButton("Copy to Clipboard");
    private JButton saveBtn = new JButton("Save Code...");
    private JTextField compNameFld = new JTextField();
    private JTextField indentFld = new JTextField();

    /**
     * Construct the code preview frame and layout the GUI.
     */
    public CodeFrame() {
        super("Code");

        codeArea.setFont(new Font("Monospaced", Font.PLAIN, 12));

        Container contentPane = getContentPane();
        JScrollPane sp = new JScrollPane();
        sp.getViewport().setView(codeArea);
        contentPane.setLayout(new GridBagLayout());

        // constraints for the scrollpane which contains the code area
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.fill = GridBagConstraints.BOTH;
        contentPane.add(sp, gbc);


        // constraints for component name label and field
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.weightx = 0;
        gbc.weighty = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        contentPane.add(new JLabel("Component Name:"), gbc);

        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        contentPane.add(compNameFld, gbc);

        // constraints for indent label and field
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.fill = GridBagConstraints.NONE;
        contentPane.add(new JLabel("Indent:"), gbc);

        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        contentPane.add(indentFld, gbc);

        JPanel buttonPanel = new JPanel(new GridLayout(1,3));
        buttonPanel.add(generateBtn);
        buttonPanel.add(copyBtn);
        buttonPanel.add(saveBtn);

        // constraints for buttonPanel
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.gridwidth = 2;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.weightx = 0;
        gbc.fill = GridBagConstraints.NONE;
        contentPane.add(buttonPanel, gbc);

        pack();

        generateBtn.addActionListener(this);
        copyBtn.addActionListener(this);
        saveBtn.addActionListener(this);
    }


    /**
     * Updates all GUI fields and generates code.
     * @param model the data source containing all of the GridBagConstraints.
     * @param codeGen the object that generates code.
     */
    public void setModel(ConstraintsModel model,
                         CodeGenerator codeGen) {
        this.model = model;
        this.codeGen = codeGen;

        compNameFld.setText(codeGen.getCompName());
        indentFld.setText(Integer.toString(codeGen.getIndent()));

        generateCode();
    }

    /**
     * Respond to the user clicking on one of the buttons.  These buttons
     * are Hide, Generate, and Copy to Clipboard.
     */
    public void actionPerformed(ActionEvent evt) {
        Object src = evt.getSource();
        if (src == generateBtn) {
            generateCode();
        } else if (src == copyBtn) {
            copyToClipboard();
        } else if (src == saveBtn) {
            saveCode();
        }
    }


    // update the text area with new source code.
    private void generateCode() {
        // set the component name
        codeGen.setCompName(compNameFld.getText().trim());

        // get the number of spaces to indent
        String indentStr = indentFld.getText().trim();
        if (indentStr.length() > 0) {
            try {
                codeGen.setIndent(Integer.parseInt(indentStr));
            } catch (NumberFormatException nfe) {
                // revert to the original value
                indentFld.setText(Integer.toString(codeGen.getIndent()));
            }
        }
        // generate code and display it
        codeArea.setText(codeGen.generateCode(model));
    }


    // copy all generated code into the system clipboard
    private void copyToClipboard() {
        StringSelection ss = new StringSelection(codeArea.getText());
        Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
        cb.setContents(ss, ss);
    }


    private void saveCode() {
        JFileChooser chooser = new JFileChooser(".");
        int returnVal = chooser.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selectedFile = chooser.getSelectedFile();
            String fullFileName = selectedFile.getPath(); 

            PrintWriter pw = null;
            try {
                pw = new PrintWriter(new FileWriter(
                    fullFileName));
                pw.println(codeArea.getText());
            } catch (Exception ex) {
                GridBagExplorer.showErrorDialog(this,
                    "Unable to save file:\n" + ex.getMessage());
            } finally {
                if (pw != null) {
                    pw.close();
                }
            }
        }
    }
}

