package com.showmejava.gridbag;

import java.awt.Insets;
import java.io.*;


/**
 * Generates code for GridBagExplorer.  This class allows the user to specify
 * the indentation and the name of the component used in the generated code.
 * @author Eric M. Burke
 * @version 2.0
 */
public class CodeGenerator {
    private int indent;
    private String compName;

    /**
     * Construct a new code generator object.
     */
    public CodeGenerator() {
        reset();
    }

    /**
     * Used with File...New to reset all fields to initial values.
     */
    public void reset() {
        indent = 0;
        compName = "";
    }

    /** 
     * Specify the indentation of the generated code.
     * @param indent the number of spaces to indent each line of code.
     */
    public void setIndent(int indent) {
        this.indent = indent;
    }

    /**
     * Return the indentation.
     * @return the number of spaces to indent each line of code.
     */
    public int getIndent() {
        return indent;
    }

    /**
     * Specify the component name.
     * @param compName the component name used in the generated code.  This
     * defaults to an empty string.
     */
    public void setCompName(String compName) {
        this.compName = compName;
    }

    /**
     * Return the component name.
     * @return the component name used in the generated code.
     */
    public String getCompName() {
        return compName;
    }

    /**
     * Generate source code.
     * @param model the data model to generate code for.
     * @return the complete source code for this layout.
     */
    public String generateCode(ConstraintsModel model) {
        if (model == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();
        appendln(buf, "// code generated by GridBagExplorer " +
            GridBagExplorer.getVersion());
        appendln(buf, "// Copyright (C) 1998, Eric M. Burke");
        appendln(buf, "// http://www.showmejava.com/");
        appendln(buf, "// ericb@showmejava.com\n");

        if (compName.length() > 0) {
            appendln(buf, compName + ".setLayout(new GridBagLayout());");
        } else {
            appendln(buf, "setLayout(new GridBagLayout());");
        }
        appendln(buf, "GridBagConstraints gbc = new GridBagConstraints();");

        Constraints prevC = new Constraints();
        int numConstraints = model.getSize();
        for (int i = 0; i < numConstraints; i++) {
            Constraints curC = model.getConstraintsAt(i);
            buf.append("\n");
            appendln(buf, "// constraints for " + curC.getName());

            // tbd: also have a generateVerboseCode option ?
            generateDiffCode(buf, prevC, curC);

            if (compName.length() > 0) {
                appendln(buf, compName + ".add(" + curC.getName() + ", gbc);");
            } else {
                appendln(buf, "add(" + curC.getName() + ", gbc);");
            }
            prevC = curC;
        }

        return buf.toString();
    }


    /**
     * Write the state of this object to a PrintWriter.   This is used
     * in the File...Save operation.
     * @param pw the object to print to.
     */
    public void write(PrintWriter pw) throws IOException {
        pw.println(indent);
        pw.println(compName);
    }
    
    /** 
     * Read this object from a BufferedReader.
     * @param br the reader to get data from.
     * @see #write(PrintWriter)
     */
    public void read(BufferedReader br) throws IOException {
        try {
            indent = Integer.parseInt(br.readLine().trim());
            compName = br.readLine().trim();
        } catch (NumberFormatException nfe) {
            throw new IOException(nfe.getMessage());
        }
    }

    // generate a fragment of code which contains the difference between
    // the previous and current Constraints objects
    private void generateDiffCode(StringBuffer buf,
                                  Constraints prev,
                                  Constraints cur) {
        String gridx = cur.getGridX();
        if (!gridx.equals(prev.getGridX())) {
            // if gridx is RELATIVE, prepend the value
            // with GridBagConstraints.
            if (gridx.startsWith("R")) {
                gridx = "GridBagConstraints." + gridx;
            }
            appendln(buf, "gbc.gridx = " + gridx + ";");
        }

        String gridy = cur.getGridY();
        if (!gridy.equals(prev.getGridY())) {
            // if gridy is RELATIVE, prepend the value
            // with GridBagConstraints.
            if (gridy.startsWith("R")) {
                gridy = "GridBagConstraints." + gridy;
            }
            appendln(buf, "gbc.gridy = " + gridy + ";");
        }

        String gridwidth = cur.getGridWidth();
        if (!gridwidth.equals(prev.getGridWidth())) {
            // if gridwidth is RELATIVE or REMAINDER, prepend the value
            // with GridBagConstraints.
            if (gridwidth.startsWith("R")) {
                gridwidth = "GridBagConstraints." + gridwidth;
            }
            appendln(buf, "gbc.gridwidth = " + gridwidth + ";");
        }

        String gridheight = cur.getGridHeight();
        if (!gridheight.equals(prev.getGridHeight())) {
            // if gridheight is RELATIVE or REMAINDER, prepend the value
            // with GridBagConstraints.
            if (gridheight.startsWith("R")) {
                gridheight = "GridBagConstraints." + gridheight;
            }
            appendln(buf, "gbc.gridheight = " + gridheight + ";");
        }

        String weightx = cur.getWeightX();
        if (!weightx.equals(prev.getWeightX())) {
            appendln(buf, "gbc.weightx = " + weightx + ";");
        }

        String weighty = cur.getWeightY();
        if (!weighty.equals(prev.getWeightY())) {
            appendln(buf, "gbc.weighty = " + weighty + ";");
        }

        String anchor = cur.getAnchor();
        if (!anchor.equals(prev.getAnchor())) {
            appendln(buf, "gbc.anchor = GridBagConstraints." + anchor + ";");
        }

        String fill = cur.getFill();
        if (!fill.equals(prev.getFill())) {
            appendln(buf, "gbc.fill = GridBagConstraints." + fill + ";");
        }

        Insets curInsets = cur.getInsets();
        Insets prevInsets = prev.getInsets();
        if (curInsets.top != prevInsets.top) {
            appendln(buf, "gbc.insets.top = " + curInsets.top + ";");
        }
        if (curInsets.left != prevInsets.left) {
            appendln(buf, "gbc.insets.left = " + curInsets.left + ";");
        }
        if (curInsets.bottom != prevInsets.bottom) {
            appendln(buf, "gbc.insets.bottom = " + curInsets.bottom + ";");
        }
        if (curInsets.right != prevInsets.right) {
            appendln(buf, "gbc.insets.right = " + curInsets.right + ";");
        }

        String ipadx = cur.getIPadX();
        if (!ipadx.equals(prev.getIPadX())) {
            appendln(buf, "gbc.ipadx = " + ipadx + ";");
        }

        String ipady = cur.getIPadY();
        if (!ipady.equals(prev.getIPadY())) {
            appendln(buf, "gbc.ipady = " + ipady + ";");
        }
    }


    // indent the appropriate number of spaces and append a line of text
    // to the StringBuffer
    private void appendln(StringBuffer buf, String str) {
        for (int i = 0; i < indent; i++) {
            buf.append(" ");
        }
        buf.append(str).append("\n");
    }
}

