package com.showmejava.gridbag;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import java.io.Serializable;

/**
 * Wraps around a GridBagConstraints object.  The purpose of 
 * this class is to provide notification when any field is changed.  It also
 * uses Strings for every field in order to simplify the GUI code.  This
 * class uses JavaBeans bound properties for notification.
 * @author Eric M. Burke
 * @version 2.0
 */
public class Constraints implements Serializable {
    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String NAME_PROP = "name";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String GRIDX_PROP = "gridX";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String GRIDY_PROP = "gridY";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String GRIDWIDTH_PROP = "gridWidth";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String GRIDHEIGHT_PROP = "gridHeight";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String WEIGHTX_PROP = "weightX";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String WEIGHTY_PROP = "weightY";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String ANCHOR_PROP = "anchor";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String FILL_PROP = "fill";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String INSETS_BOTTOM_PROP = "insetsBottom";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String INSETS_LEFT_PROP = "insetsLeft";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String INSETS_RIGHT_PROP = "insetsRight";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String INSETS_TOP_PROP = "insetsTop";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String IPADX_PROP = "IPadX";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String IPADY_PROP = "IPadY";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String PREFSIZEWIDTH_PROP = "prefSizeWidth";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String PREFSIZEHEIGHT_PROP = "prefSizeHeight";

    /**
     * The name of a bound property for JavaBeans.
     */
    public static final String USEDEFAULTPREFSIZE_PROP = "useDefaultPrefSize";

    // default values
    private static String DEF_GRIDX;
    private static String DEF_GRIDY;
    private static String DEF_GRIDWIDTH;
    private static String DEF_GRIDHEIGHT;
    private static String DEF_WEIGHTX;
    private static String DEF_WEIGHTY;
    private static String DEF_ANCHOR;
    private static String DEF_FILL;
    private static String DEF_INSETS_BOTTOM;
    private static String DEF_INSETS_LEFT;
    private static String DEF_INSETS_RIGHT;
    private static String DEF_INSETS_TOP;
    private static String DEF_IPADX;
    private static String DEF_IPADY;

    // compute default string values
    static {
        GridBagConstraints gbc = new GridBagConstraints();
        try {
            DEF_GRIDX = GBCUtil.gridXOrYToString(gbc.gridx);
            DEF_GRIDY = GBCUtil.gridXOrYToString(gbc.gridy);
            DEF_GRIDWIDTH = GBCUtil.gridWidthOrHeightToString(gbc.gridwidth);
            DEF_GRIDHEIGHT = GBCUtil.gridWidthOrHeightToString(gbc.gridheight);
            DEF_WEIGHTX = GBCUtil.weightXOrYToString(gbc.weightx);
            DEF_WEIGHTY = GBCUtil.weightXOrYToString(gbc.weighty);
            DEF_ANCHOR = GBCUtil.anchorToString(gbc.anchor);
            DEF_FILL = GBCUtil.fillToString(gbc.fill);
            DEF_INSETS_BOTTOM = GBCUtil.insetsFieldToString(gbc.insets.bottom);
            DEF_INSETS_LEFT = GBCUtil.insetsFieldToString(gbc.insets.left);
            DEF_INSETS_RIGHT = GBCUtil.insetsFieldToString(gbc.insets.right);
            DEF_INSETS_TOP = GBCUtil.insetsFieldToString(gbc.insets.top);
            DEF_IPADX = GBCUtil.iPadXOrYToString(gbc.ipadx);
            DEF_IPADY = GBCUtil.iPadXOrYToString(gbc.ipady);
        } catch (ConstraintsException ce) {
            // this will never happen
        }
    }

    private GridBagConstraints gbc = new GridBagConstraints();
    private transient PropertyChangeSupport changes;
    private String name;
    private Dimension prefSize = new Dimension();
    private boolean useDefaultPrefSize = true;

    // although these values could be looked up using GBCUtil, storing
    // the values here is faster when the GUI needs to retrieve the
    // values.

    // set to default values
    private String gridx        = DEF_GRIDX;
    private String gridy        = DEF_GRIDY;
    private String gridwidth    = DEF_GRIDWIDTH;
    private String gridheight   = DEF_GRIDHEIGHT;
    private String weightx      = DEF_WEIGHTX;
    private String weighty      = DEF_WEIGHTY;
    private String anchor       = DEF_ANCHOR;
    private String fill         = DEF_FILL;
    private String insetsTop    = DEF_INSETS_TOP;
    private String insetsBottom = DEF_INSETS_BOTTOM;
    private String insetsRight  = DEF_INSETS_RIGHT;
    private String insetsLeft   = DEF_INSETS_LEFT;
    private String ipadx        = DEF_IPADX;
    private String ipady        = DEF_IPADY;


    /**
     * Construct a new constraints object with the name defaulted to
     * an empty String.  This constructor is provided for compatability
     * with JavaBeans.
     */
    public Constraints() {
        this("");
    }

    /**
     * Construct a new constraints object with the specified name.
     * @param name the name for this constraints.
     */
    public Constraints(String name) {
        this.name = name;
        init();
    }


    /**
     * Register a PropertyChangeListener.  All "set" methods use bound
     * properties and fire PropertyChange events.  The list of listeners
     * is transient, so de-serializing this object will necessitate
     * re-registration of listeners.
     * @param l the listener to register.
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        changes.addPropertyChangeListener(l);
    }

    /**
     * Un-register a PropertyChangeListener.
     * @param l the listener to remove.
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        changes.removePropertyChangeListener(l);
    }

    /**
     * Specify a new name.  This is a bound property.
     * @param name the new name.
     */
    public synchronized void setName(String name) {
        String oldName = this.name;
        this.name = name;
        changes.firePropertyChange(NAME_PROP, oldName, name);
    }


    /**
     * Create and return a clone of the internal GridBagConstraints
     * object.  A clone is returned rather than a reference to the
     * internal object, otherwise the caller might modify fields in
     * the constraints, bypassing event notification.
     * @return a clone of the internal GridBagConstraints object.
     */
    public GridBagConstraints getGridBagConstraints() {
        return (GridBagConstraints)gbc.clone();
    }

    /**
     * Set the value of gridx.
     * @param gridx the new value for gridx.
     * @throws ConstraintsException if the value of gridx is illegal.
     */
    public synchronized void setGridX(String gridx) 
    throws ConstraintsException {
        String oldGridX = this.gridx;
        gbc.gridx = GBCUtil.stringToGridXOrY(gridx);
        this.gridx = gridx;
        changes.firePropertyChange(GRIDX_PROP, oldGridX, gridx);
    }

    /**
     * Set the value of gridy.
     * @param gridy the new value for gridy.
     * @throws ConstraintsException if the value of gridy is illegal.
     */
    public synchronized void setGridY(String gridy) 
    throws ConstraintsException {
        String oldGridY = this.gridy;
        gbc.gridy = GBCUtil.stringToGridXOrY(gridy);
        this.gridy = gridy;
        changes.firePropertyChange(GRIDY_PROP, oldGridY, gridy);
    }

    /**
     * Set the value of gridwidth.
     * @param gridwidth the new value for gridwidth.
     * @throws ConstraintsException if the value of gridwidth is illegal.
     */
    public synchronized void setGridWidth(String gridwidth) 
    throws ConstraintsException {
        String oldGridWidth = this.gridwidth;
        gbc.gridwidth = GBCUtil.stringToGridWidthOrHeight(gridwidth);
        this.gridwidth = gridwidth;
        changes.firePropertyChange(GRIDWIDTH_PROP, oldGridWidth, gridwidth);
    }

    /**
     * Set the value of gridheight.
     * @param gridheight the new value for gridheight.
     * @throws ConstraintsException if the value of gridheight is illegal.
     */
    public synchronized void setGridHeight(String gridheight) 
    throws ConstraintsException {
        String oldGridHeight = this.gridheight;
        gbc.gridheight = GBCUtil.stringToGridWidthOrHeight(gridheight);
        this.gridheight = gridheight;
        changes.firePropertyChange(GRIDHEIGHT_PROP, oldGridHeight, gridheight);
    }

    /**
     * Set the value of weightx.
     * @param weightx the new value for weightx.
     * @throws ConstraintsException if the value of weightx is illegal.
     */
    public synchronized void setWeightX(String weightx)
    throws ConstraintsException {
        String oldWeightX = this.weightx;
        gbc.weightx = GBCUtil.stringToWeightXOrY(weightx);
        this.weightx = weightx;
        changes.firePropertyChange(WEIGHTX_PROP, oldWeightX, weightx);
    }


    /**
     * Set the value of weighty.
     * @param weighty the new value for weighty.
     * @throws ConstraintsException if the value of weighty is illegal.
     */
    public synchronized void setWeightY(String weighty)
    throws ConstraintsException {
        String oldWeightY = this.weighty;
        gbc.weighty = GBCUtil.stringToWeightXOrY(weighty);
        this.weighty = weighty;
        changes.firePropertyChange(WEIGHTY_PROP, oldWeightY, weighty);
    }


    /**
     * Set the value of anchor.
     * @param anchor the new value for anchor.
     * @throws ConstraintsException if the value of anchor is illegal.
     */
    public synchronized void setAnchor(String anchor)
    throws ConstraintsException {
        String oldAnchor = this.anchor;
        gbc.anchor = GBCUtil.stringToAnchor(anchor);
        this.anchor = anchor;
        changes.firePropertyChange(ANCHOR_PROP, oldAnchor, anchor);
    }


    /**
     * Set the value of fill.
     * @param fill the new value for fill.
     * @throws ConstraintsException if the value of fill is illegal.
     */
    public synchronized void setFill(String fill) 
    throws ConstraintsException {
        String oldFill = this.fill;
        gbc.fill = GBCUtil.stringToFill(fill);
        this.fill = fill;
        changes.firePropertyChange(FILL_PROP, oldFill, fill);
    }

    /**
     * Set the value of insets.top.
     * @param insetsTop the new value for insets.top.
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setInsetsTop(String insetsTop) 
    throws ConstraintsException {
        String oldInsetsTop = this.insetsTop;
        gbc.insets.top = GBCUtil.stringToInsetsField(insetsTop);
        this.insetsTop = insetsTop;
        changes.firePropertyChange(INSETS_TOP_PROP, oldInsetsTop, insetsTop);
    }

    /**
     * Set the value of insets.left.
     * @param insetsLeft the new value for insets.left.
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setInsetsLeft(String insetsLeft)
    throws ConstraintsException {
        String oldInsetsLeft = this.insetsLeft;
        gbc.insets.left = GBCUtil.stringToInsetsField(insetsLeft);
        this.insetsLeft = insetsLeft;
        changes.firePropertyChange(INSETS_LEFT_PROP, oldInsetsLeft, insetsLeft);
    }

    /**
     * Set the value of insets.bottom.
     * @param insetsBottom the new value for insets.bottom.
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setInsetsBottom(String insetsBottom)
    throws ConstraintsException {
        String oldInsetsBottom = this.insetsBottom;
        gbc.insets.bottom = GBCUtil.stringToInsetsField(insetsBottom);
        this.insetsBottom = insetsBottom;
        changes.firePropertyChange(INSETS_BOTTOM_PROP, oldInsetsBottom, 
                insetsBottom);
    }

    /**
     * Set the value of insets.right.
     * @param insetsRight the new value for insets.right.
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setInsetsRight(String insetsRight)
    throws ConstraintsException {
        String oldInsetsRight = this.insetsRight;
        gbc.insets.right = GBCUtil.stringToInsetsField(insetsRight);
        this.insetsRight = insetsRight;
        changes.firePropertyChange(INSETS_RIGHT_PROP, oldInsetsRight,
                insetsRight);
    }

    /**
     * Set the value of ipadx.
     * @param ipadx the new value for ipadx.
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setIPadX(String ipadx) 
    throws ConstraintsException {
        String oldIPadX = this.ipadx;
        gbc.ipadx = GBCUtil.stringToIPadXOrY(ipadx);
        this.ipadx = ipadx;
        changes.firePropertyChange(IPADX_PROP, oldIPadX, ipadx);
    }

    /**
     * Set the value of ipady.
     * @param ipady the new value for ipady.
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setIPadY(String ipady) 
    throws ConstraintsException {
        String oldIPadY = this.ipady;
        gbc.ipady = GBCUtil.stringToIPadXOrY(ipady);
        this.ipady = ipady;
        changes.firePropertyChange(IPADY_PROP, oldIPadY, ipady);
    }

    /**
     * Set the value of preferredSize.width.
     * @param prefSizeWidth the new value for preferredSize.width
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setPrefSizeWidth(String prefSizeWidth) 
    throws ConstraintsException {
        try {
            String oldPrefSizeWidth = Integer.toString(prefSize.width);
            prefSize.width = Integer.parseInt(prefSizeWidth);
            changes.firePropertyChange(PREFSIZEWIDTH_PROP,
                    oldPrefSizeWidth, prefSizeWidth);
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal preferred width");
        }
    }

    /**
     * Set the value of preferredSize.height.
     * @param prefSizeHeight the new value for preferredSize.height
     * @throws ConstraintsException if the value is illegal.
     */
    public synchronized void setPrefSizeHeight(String prefSizeHeight) 
    throws ConstraintsException {
        try {
            String oldPrefSizeHeight = Integer.toString(prefSize.height);
            prefSize.height = Integer.parseInt(prefSizeHeight);
            changes.firePropertyChange(PREFSIZEHEIGHT_PROP,
                    oldPrefSizeHeight, prefSizeHeight);
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal preferred height");
        }
    }

    /**
     * @return the name of this object.
     */
    public String getName() {
        return name;
    }

    /**
     * Get the value of gridx.
     * @return the value of gridx.
     */
    public String getGridX() {
        return gridx;
    }

    /**
     * Get the value of gridy.
     * @return the value of gridy.
     */
    public String getGridY() {
        return gridy;
    }

    /**
     * Get the value of gridwidth.
     * @return the value of gridwidth.
     */
    public String getGridWidth() {
        return gridwidth;
    }

    /**
     * Get the value of gridheight.
     * @return the value of gridheight.
     */
    public String getGridHeight() {
        return gridheight;
    }

    /**
     * Get the value of weightx.
     * @return the value of weightx.
     */
    public String getWeightX() {
        return weightx;
    }

    /**
     * Get the value of weighty.
     * @return the value of weighty.
     */
    public String getWeightY() {
        return weighty;
    }

    /**
     * Get the value of anchor.
     * @return the value of anchor.
     */
    public String getAnchor() {
        return anchor;
    }

    /**
     * Get the value of fill.
     * @return the value of fill.
     */
    public String getFill() {
        return fill;
    }

    /**
     * Get the value of insets.
     * @return the value of insets.
     */
    public Insets getInsets() {
        return (Insets)gbc.insets.clone();
    }

    /**
     * Get the value of insets.top.
     * @return the value of insets.top.
     */
    public String getInsetsTop() {
        return insetsTop;
    }

    /**
     * Get the value of insets.left.
     * @return the value of insets.left.
     */
    public String getInsetsLeft() {
        return insetsLeft;
    }

    /**
     * Get the value of insets.bottom.
     * @return the value of insets.bottom.
     */
    public String getInsetsBottom() {
        return insetsBottom;
    }

    /**
     * Get the value of insets.right.
     * @return the value of insets.right.
     */
    public String getInsetsRight() {
        return insetsRight;
    }

    /**
     * Get the value of ipadx.
     * @return the value of ipadx.
     */
    public String getIPadX() {
        return ipadx;
    }

    /**
     * Get the value of ipady.
     * @return the value of ipady.
     */
    public String getIPadY() {
        return ipady;
    }

    /**
     * Get the string representation of this object.
     * @return the name of this constraints.
     */
    public String toString() {
        return name;
    }


    /**
     * @return the explicitly set preferred width of this component.
     */
    public String getPrefSizeWidth() {
        return Integer.toString(prefSize.width);
    }

    /**
     * @return the explicitly set preferred height of this component.
     */
    public String getPrefSizeHeight() {
        return Integer.toString(prefSize.height);
    }

    /**
     * @return a clone of the prefSize field.
     */
    public Dimension getPrefSize() {
        return new Dimension(prefSize);
    }

    /**
     * @param useDefaultPrefSize if true, use the components preferred
     * size rather than the explicit preferred size.
     * @see setPreferredSize
     */
    public synchronized void setUseDefaultPrefSize(
            boolean useDefaultPrefSize) {
        boolean oldUseDefaultPrefSize = this.useDefaultPrefSize;
        this.useDefaultPrefSize = useDefaultPrefSize;
        changes.firePropertyChange(USEDEFAULTPREFSIZE_PROP,
                new Boolean(oldUseDefaultPrefSize),
                new Boolean(useDefaultPrefSize));
    }

    /**
     * @return the flag indicating whether the component's default
     * preferred size should be used.
     */
    public boolean getUseDefaultPrefSize() {
        return useDefaultPrefSize;
    }

    /**
     * Write the state of this object to a PrintWriter.  This is used in
     * the File...Save operation.
     * @param pw the object to write to.
     */
    public void write(PrintWriter pw) {
        pw.println(getName());
        pw.println(getGridX());
        pw.println(getGridY());
        pw.println(getGridWidth());
        pw.println(getGridHeight());
        pw.println(getWeightX());
        pw.println(getWeightY());
        pw.println(getAnchor());
        pw.println(getFill());
        pw.println(getInsetsTop());
        pw.println(getInsetsLeft());
        pw.println(getInsetsBottom());
        pw.println(getInsetsRight());
        pw.println(getIPadX());
        pw.println(getIPadY());
        pw.println(prefSize.width);
        pw.println(prefSize.height);
        pw.println(useDefaultPrefSize);
    }

    /**
     * Read from a BufferedReader.
     * @param br the object to read from.
     * @throws IOException if anything goes wrong.
     */
    public void read(BufferedReader br) throws IOException {
        try {
            setName(br.readLine());
            setGridX(br.readLine());
            setGridY(br.readLine());
            setGridWidth(br.readLine());
            setGridHeight(br.readLine());
            setWeightX(br.readLine());
            setWeightY(br.readLine());
            setAnchor(br.readLine());
            setFill(br.readLine());
            setInsetsTop(br.readLine());
            setInsetsLeft(br.readLine());
            setInsetsBottom(br.readLine());
            setInsetsRight(br.readLine());
            setIPadX(br.readLine());
            setIPadY(br.readLine());

            try {
                int width = Integer.parseInt(br.readLine());
                int height = Integer.parseInt(br.readLine());
                prefSize = new Dimension(width, height);
                useDefaultPrefSize = Boolean.valueOf(
                        br.readLine()).booleanValue();
            } catch (NumberFormatException nfe) {
                prefSize = null;
            }
        } catch (ConstraintsException ce) {
            throw new IOException(ce.getMessage());
        }
    }

    // initialize transient fields
    private void init() {
        changes = new PropertyChangeSupport(this);
    }

    // overridden in order to call the init() method
    private void readObject(ObjectInputStream is) throws 
    ClassNotFoundException, IOException {
        is.defaultReadObject();
        init();
    }
}

