package com.showmejava.gridbag;

import javax.swing.*;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

/**
 * A data structure which holds a list of Constraints objects.  This class
 * also implements the AbstractListModel interface, allowing it to be
 * attached to a JList directly.
 * @author Eric M. Burke
 * @version 2.0
 */
public class ConstraintsModel extends AbstractListModel 
implements PropertyChangeListener {
    private Vector constraints = new Vector();

    /**
     * Get the size of this model.
     * @return the number of constraints in this model.
     */
    public int getSize() {
        return constraints.size();
    }

    /**
     * This method is required by the AbstractListModel class.  This
     * returns the exact same thing as getConstraintsAt.
     * @param index the location of the constraints to get.
     * @return the Constraints object at the specified index.
     */
    public Object getElementAt(int index) {
        return constraints.elementAt(index);
    }

    /**
     * Get a Constraints from the specified index.
     * @param index the index of the Constraints to get.
     * @return a reference to one of the Constraints objects.
     */
    public synchronized Constraints getConstraintsAt(int index) {
        return (Constraints)getElementAt(index);
    }


    /**
     * Implement the PropertyChangeListener interface to properly notify
     * the list whenever any one of the constraints objects changes.
     * @param evt the PropertyChangeEvent from one of the constraints.
     */
    public void propertyChange(PropertyChangeEvent evt) {
        Object src = evt.getSource();
        int index = constraints.indexOf(src);
        if (index > -1) {
            fireContentsChanged(this, index, index);
        }
    }

    /**
     * Add a new Constraints object.
     * @param c the object to add to the end of the list.
     */
    public synchronized void addConstraints(Constraints c) {
        c.addPropertyChangeListener(this);
        constraints.addElement(c);
        fireIntervalAdded(this, 
                          constraints.size()-1,
                          constraints.size()-1);
    } 

    /**
     * Insert a new Constraints object at the specified index.
     * @param c the object to add.
     * @param index the location in the list to add.
     */
    public synchronized void insertConstraintsAt(Constraints c, int index) {
        c.addPropertyChangeListener(this);
        constraints.insertElementAt(c, index);
        fireIntervalAdded(this, 
                          index,
                          index);
    }

    /**
     * Support the File...New operation by removing all Constraints.
     */
    public synchronized void reset() {
        int numConstraints = constraints.size();
        if (numConstraints > 0) {
            for (int i = 0; i < numConstraints; i++) {
                Constraints c = (Constraints)constraints.elementAt(i);
                c.removePropertyChangeListener(this);
            }
            constraints.removeAllElements();
            fireIntervalRemoved(this, 0, numConstraints-1);
        }
    }

    /**
     * Remove a single Constraints.
     * @param i the index of the Constraints to remove
     * @return the object which was removed.
     */
    public synchronized Constraints removeConstraintsAt(int i) {
        return removeConstraints((Constraints)constraints.elementAt(i));
    }


    /**
     * Remove a single Constraints.
     * @param c the object to remove.
     * @return the object which was removed.
     */
    public synchronized Constraints removeConstraints(Constraints c) {
        if (constraints.contains(c)) {
            int index = constraints.indexOf(c);
            constraints.removeElementAt(index);
            c.removePropertyChangeListener(this);
            fireIntervalRemoved(this,
                                index,
                                index);
        }
        return c;
    }


    /**
     * Check for existing component name to prevent duplicates.
     * @param compName the name to search for.
     * @return true if the component name is already used.
     */
    public synchronized boolean isNameTaken(String compName) {
        int numConstraints = constraints.size();
        for (int i = 0; i < numConstraints; i++) {
            Constraints c = (Constraints)constraints.elementAt(i);
            if (c.getName().equals(compName)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Store this object to a PrintWriter.
     * @param pw the PrintWriter to write to.
     * @throws IOException if the write failed.
     */
    public void write(PrintWriter pw) throws IOException {
        int numConstraints = constraints.size();

        pw.println(numConstraints);
        for (int i = 0; i < numConstraints; i++) {
            ((Constraints)constraints.elementAt(i)).write(pw);
        }
    }
    
    /**
     * Read this object from a BufferedReader.
     * @param br the object to read from.
     * @throws IOException if anything goes wrong.
     */
    public void read(BufferedReader br) throws IOException {
        try {
            int numConstraints = Integer.parseInt(
                    br.readLine());
            Vector vec = new Vector(numConstraints);
            for (int i = 0; i < numConstraints; i++) {
                Constraints c = new Constraints();
                c.read(br);
                c.addPropertyChangeListener(this);
                vec.addElement(c);
            }

            reset();
            this.constraints = vec;
            fireIntervalAdded(this, 
                              0,
                              numConstraints-1);
        } catch (NumberFormatException nfe) {
            throw new IOException("The file did not contain the number"
                    + " of constraints.");
        }
    }
}

