package com.showmejava.gridbag;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.event.*;
import java.awt.*;
import java.io.*;

/**
 * The main window and the entry point for the entire application.
 * @author Eric M. Burke
 * @version 2.0
 */
public class GridBagExplorer extends JFrame implements ActionListener,
ListSelectionListener, DocumentListener, ListDataListener {
    private static final String TITLE = "GridBagExplorer";
    private static boolean errorDialogVisible;
    private ConstraintsEditorPanel editorPanel = new ConstraintsEditorPanel();
    private CodeFrame codeFrame;
    private PreviewFrame previewFrame;
    private ConstraintsModel constraintsModel = new ConstraintsModel();
    private CodeGenerator codeGen = new CodeGenerator();
    private JList constraintsList = new JList(constraintsModel);
    private JButton upBtn = new JButton("Up");
    private JButton downBtn = new JButton("Down");
    private JButton removeBtn = new JButton("Remove");
    private JButton addBtn = new JButton("Add");
    private JTextField addFld = new JTextField();

    private JMenuItem exitItem = new JMenuItem("Exit");
    private JMenuItem openItem = new JMenuItem("Open...");
    private JMenuItem newItem = new JMenuItem("New");
    private JMenuItem saveItem = new JMenuItem("Save");
    private JMenuItem saveAsItem = new JMenuItem("Save As...");
    private JMenuItem previewItem = new JMenuItem("Preview...");
    private JMenuItem codeItem = new JMenuItem("Code...");
    private JMenuItem helpAboutItem = new JMenuItem("About...");
    private JMenuItem[] lafMenus;

    private String fullFileName;
    private String fileName;

    private static UIManager.LookAndFeelInfo[] lafInfo = 
            UIManager.getInstalledLookAndFeels();

    /**
     * The entry point for the application.
     */
    public static void main(String[] args) {
        new GridBagExplorer().show();
    }

    public GridBagExplorer() {
        Container contentPane = getContentPane();
        contentPane.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();

        // Up button
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        contentPane.add(upBtn, gbc);

        // Down button
        gbc.gridy = 1;
        contentPane.add(downBtn, gbc);

        // Remove button
        gbc.gridy = 2;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        contentPane.add(removeBtn, gbc);

        // Add button
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        contentPane.add(addBtn, gbc);

        // Constraints list
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridheight = 3;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        JScrollPane sp = new JScrollPane();
        sp.getViewport().setView(constraintsList);
        sp.setPreferredSize(new Dimension(150,150));
        contentPane.add(sp, gbc);

        // Add field
        gbc.gridy = 3;
        gbc.gridheight = 1;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        contentPane.add(addFld, gbc);

        // Constraints editor
        gbc.weightx = 0;
        gbc.gridx = 2;
        gbc.gridy = 0;
        gbc.gridheight = 5;
        gbc.fill = GridBagConstraints.NONE;
        contentPane.add(editorPanel, gbc);

        // menus
        JMenu fileMenu = new JMenu("File");
        fileMenu.add(newItem);
        fileMenu.add(openItem);
        fileMenu.add(saveItem);
        fileMenu.add(saveAsItem);
        fileMenu.addSeparator();
        fileMenu.add(exitItem);

        JMenu windowMenu = new JMenu("Window");
        windowMenu.add(previewItem);
        windowMenu.add(codeItem);

        JMenu helpMenu = new JMenu("Help");
        helpMenu.add(helpAboutItem);

        JMenuBar mb = new JMenuBar();
        mb.add(fileMenu);
        mb.add(windowMenu);
        mb.add(helpMenu);
        // tbd: this isn't implemented in the current version of Swing...odd
        // mb.setHelpMenu(helpMenu);
        setJMenuBar(mb);

        // set up the look & feel menu
        lafMenus = new JMenuItem[lafInfo.length];
        windowMenu.addSeparator();
        for (int i = 0; i < lafInfo.length; i++) {
            lafMenus[i] = new JMenuItem(lafInfo[i].getName());
            windowMenu.add(lafMenus[i]);
            lafMenus[i].addActionListener(this);
        }

        newItem.addActionListener(this);
        openItem.addActionListener(this);
        saveItem.addActionListener(this);
        saveAsItem.addActionListener(this);
        exitItem.addActionListener(this);
        previewItem.addActionListener(this);
        codeItem.addActionListener(this);
        helpAboutItem.addActionListener(this);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                exitApp();
            }
        });
        upBtn.addActionListener(this);
        downBtn.addActionListener(this);
        removeBtn.addActionListener(this);
        addBtn.addActionListener(this);

        constraintsList.addListSelectionListener(this);
        constraintsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        constraintsModel.addListDataListener(this);

        addFld.getDocument().addDocumentListener(this);
        addFld.addActionListener(this);

        pack();
        setSensitivities();
        updateTitle();

        addFld.setText("Start Here!");
        addFld.requestFocus();
        addFld.selectAll();
    }

    /**
     * @return the current version number of this application.
     */
    public static String getVersion() {
        // NOTE: Whenever you change this, ALSO change the code in
        // the fileOpen method to handle version number changes in
        // data files.
        return "2.0.4";
    }


    void exitApp() {
        System.exit(0);
    }

    public void changedUpdate(DocumentEvent evt) {
        setSensitivities();
    }

    public void removeUpdate(DocumentEvent evt) {
        setSensitivities();
    }

    public void insertUpdate(DocumentEvent evt) {
        setSensitivities();
    }

    public static void showErrorDialog(Component parent, String message) {
        // prevent multiple copies from being displayed
        if (!errorDialogVisible) {
            errorDialogVisible = true;
            JOptionPane.showMessageDialog(parent, message,
                    "Message", JOptionPane.ERROR_MESSAGE);
        }
        errorDialogVisible = false;
    }

    public void actionPerformed(ActionEvent evt) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            Object src = evt.getSource();
            if (src == upBtn) {
                upPressed();
            } else if (src == downBtn) {
                downPressed();
            } else if (src == removeBtn) {
                removePressed();
            } else if (src == addBtn || src == addFld) {
                addPressed();
            } else if (src == newItem) {
                fileNew();
            } else if (src == openItem) {
                fileOpen();
            } else if (src == saveItem) {
                save();
            } else if (src == saveAsItem) {
                saveAs();
            } else if (src == exitItem) {
                exitApp();
            } else if (src == previewItem) {
                if (previewFrame == null) {
                    previewFrame = new PreviewFrame();
                    previewFrame.setModel(constraintsModel);
                }
                previewFrame.show();
            } else if (src == codeItem) {
                if (codeFrame == null) {
                    codeFrame = new CodeFrame();
                    codeFrame.setModel(constraintsModel, codeGen);
                }
                codeFrame.show();
            } else if (src == helpAboutItem) {
                showHelpAbout();
            } else {
                // check the L&F menu items
                for (int i = 0; i < lafMenus.length; i++) {
                    if (src == lafMenus[i]) {
                        try {
                            UIManager.setLookAndFeel(lafInfo[i].getClassName());
                            SwingUtilities.updateComponentTreeUI(this);
                            // neccesary because some L&F's chop of components
                            pack();
                            if (codeFrame != null) {
                                SwingUtilities.updateComponentTreeUI(codeFrame);
                            }
                            if (previewFrame != null) {
                                SwingUtilities.updateComponentTreeUI(
                                        previewFrame);
                            }
                        } catch (Exception ex) {
                            showErrorDialog(this, 
                                "Error occurred while switching L&F:\n" +
                                ex.getMessage());
                        }
                        return;
                    }
                }
            }
        } finally {
            setCursor(Cursor.getDefaultCursor());
        }
    }

    public void valueChanged(ListSelectionEvent evt) {
        if (evt.getValueIsAdjusting()) {
            return;
        }

        setSensitivities();

    }

    public void contentsChanged(ListDataEvent e) {
      // no-op
    }

    public void intervalAdded(ListDataEvent e) {
        setSensitivities();
    }

    public void intervalRemoved(ListDataEvent e) {
        // the selection may have changed
        setSensitivities();
    }

    private void upPressed() {
        int selectedIndex = constraintsList.getSelectedIndex();
        Constraints c = constraintsModel.removeConstraintsAt(selectedIndex);
        constraintsModel.insertConstraintsAt(c, selectedIndex-1);
        constraintsList.setSelectedIndex(selectedIndex-1);
    }

    private void downPressed() {
        int selectedIndex = constraintsList.getSelectedIndex();
        Constraints c = constraintsModel.removeConstraintsAt(selectedIndex);
        constraintsModel.insertConstraintsAt(c, selectedIndex+1);
        constraintsList.setSelectedIndex(selectedIndex+1);
    }

    private void removePressed() {
        int selectedIndex = constraintsList.getSelectedIndex();
        if (selectedIndex > -1) {
            constraintsModel.removeConstraintsAt(selectedIndex);
        }
        setSensitivities();
    }

    private void addPressed() {
        String compName = addFld.getText().trim();
        if (compName.length() > 0) {
            if (constraintsModel.isNameTaken(compName)) {
                // display an error message
                showErrorDialog(this, 
                        "Duplicate component names are not allowed");
                addFld.selectAll();
            } else {
                addFld.setText("");
                constraintsModel.addConstraints(new Constraints(compName));
            }
        }
        addFld.requestFocus();
        setSensitivities();
    }


    /**
     * Enable/disable components.  Also set the title of the window.
     */
    private void setSensitivities() {
        int selectedIndex = constraintsList.getSelectedIndex();

        removeBtn.setEnabled(selectedIndex > -1);
        addBtn.setEnabled(addFld.getText().trim().length() > 0);

        int numConstraints = constraintsModel.getSize();
        upBtn.setEnabled(selectedIndex > 0);
        downBtn.setEnabled(selectedIndex > -1 &&
                           selectedIndex < (numConstraints-1));
        saveItem.setEnabled(fullFileName != null);

        // make sure the constraints panel is in sync
        if (selectedIndex > -1 && selectedIndex < numConstraints) {
            Constraints c = constraintsModel.getConstraintsAt(selectedIndex);
            editorPanel.setConstraints(c);
        } else {
            editorPanel.setConstraints(null);
        }
    }


    private void save() {
        if (fullFileName == null) {
            saveAs();
        }

        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new FileWriter(
                    fullFileName));
            pw.println(getVersion());
            constraintsModel.write(pw);
            codeGen.write(pw);
        } catch (Exception ex) {
            showErrorDialog(this, "Unable to save file:\n"
                   + ex.getMessage());
            ex.printStackTrace();
            fullFileName = null;
            fileName = null;
        } finally {
            if (pw != null) {
                pw.close();
            }
            pw = null;
        }

        setSensitivities();
        updateTitle();
        addFld.setText("");
    }

    private void saveAs() {
        // the JFileChooser is causing repainting of the menus to get 
        // screwed up.  Forcing a repaint() here seems to fix the bug.
        // this is documented on the JavaSoft bug parade under ID 4189244,
        // and has been fixed in Swing 1.1.1 beta 1
        repaint();
        JFileChooser chooser = createChooser();
        int returnVal = chooser.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selectedFile = chooser.getSelectedFile();
            fullFileName = selectedFile.getPath();
            fileName = selectedFile.getName();

            if (!fullFileName.toLowerCase().endsWith(".gbe")) {
                fullFileName = fullFileName + ".gbe";
                fileName = fileName + ".gbe";
            }
            save();
        }
    }

    private void fileNew() {
        constraintsModel.reset();
        codeGen.reset();
        // tbd: force the codeFrame to update since it isn't a listener on
        // the constraints model
        if (codeFrame != null) {
            codeFrame.setModel(constraintsModel, codeGen);
        }

        fullFileName = null;
        fileName = null;
        setSensitivities();
        updateTitle();
        addFld.setText("Start Here!");
        addFld.requestFocus();
    }

    private void showHelpAbout() {
        JOptionPane.showMessageDialog(this,
            "GridBagExplorer version " + getVersion() + "\n" +
            "Written by Eric Burke, http://www.showmejava.com/",
            "About", 
            JOptionPane.INFORMATION_MESSAGE);
    }


    private void fileOpen() {
        // tbd: the JFileChooser is causing repainting
        // of the menus to get screwed up.  Forcing a repaint() here 
        // seems to fix the bug
        // this is documented on the JavaSoft bug parade under ID 4189244
        repaint();
        JFileChooser chooser = createChooser();
        int returnVal = chooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selectedFile = chooser.getSelectedFile();
            fullFileName = selectedFile.getPath();
            fileName = selectedFile.getName();

            BufferedReader br = null;
            try {
                br = new BufferedReader(new FileReader(fullFileName));
                String fileVersion = br.readLine();
                // the format of the data file did not change for these versions
                if (!fileVersion.equals("2.0.1") && 
                    !fileVersion.equals("2.0.2") &&
                    !fileVersion.equals("2.0.3") &&
                    !fileVersion.equals("2.0.4")) {

                    showErrorDialog(this, "Unable to read file:\n"
                            + "Version number does not match.");
                    fullFileName = null;
                    fileName = null;
                } else {
                    constraintsModel.read(br);
                    codeGen.read(br);
                    // tbd: force the codeFrame to update since it isn't a 
                    // listener on the constraints model
                    if (codeFrame != null) {
                        codeFrame.setModel(constraintsModel, codeGen);
                    }
                }
            } catch (Exception ex) {
                showErrorDialog(this, "Unable to read file:\n"
                       + ex.getMessage());
                ex.printStackTrace();
                fullFileName = null;
                fileName = null;
            } finally {
                try {
                    if (br != null) {
                        br.close();
                    }
                } catch (IOException ioe) {
                }
                br = null;
                updateTitle();
                setSensitivities();
                addFld.setText("");
            }
        }
    }

    private JFileChooser createChooser() {
        JFileChooser chooser = new JFileChooser(".");
        chooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory()
                        || f.getName().toLowerCase().endsWith(".gbe");
            }

            public String getDescription() {
                return "*.gbe (GridBagExplorer data files)";
            }
        });

        return chooser;
    }

    private void updateTitle() {
        setTitle(TITLE + " " + ((fileName == null) ? "[NO FILE]" : 
            "- " + fileName));
    }
}

