package com.showmejava.gridbag;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/**
 * Displays a preview of the actual layout using JButtons for each
 * component.
 * @author Eric M. Burke
 * @version 2.0
 */
public class PreviewFrame extends JFrame
implements ListDataListener, ActionListener {

    private GridBagLayout layout = new GridBagLayout();

    private ConstraintsModel model;
    private Vector componentVec = new Vector();
    private JRootPane rootPane = new JRootPane();
    private Container layoutPanel = rootPane.getContentPane();
    private JButton packBtn = new JButton("Pack");
    private GlassPane glassPane = new GlassPane();
    private JToggleButton visibleGridBtn = new JToggleButton("Grid");
    private boolean gridVisible;

    public PreviewFrame() {
        super("Preview");

        layoutPanel.setLayout(layout);
        rootPane.setGlassPane(glassPane);

        Container contentPane = getContentPane();

        contentPane.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        // constraints for rootPane
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.fill = GridBagConstraints.BOTH;
        contentPane.add(rootPane, gbc);


        // constraints for packBtn
        gbc.fill = GridBagConstraints.NONE;
        gbc.weighty = 0;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridwidth = 1;
        contentPane.add(packBtn, gbc);

        // constraints for visibleGridBtn
        gbc.gridx = 1;
        gbc.anchor = GridBagConstraints.WEST;
        contentPane.add(visibleGridBtn, gbc);

        //////////////////////////////////////////////////////////////////////

        setSize(300,300);

        packBtn.addActionListener(this);
        visibleGridBtn.addActionListener(this);

        glassPane.setVisible(true);
    }

    public void setModel(ConstraintsModel model) {
        if (model == this.model) {
            return;
        }

        // unregister from current model
        if (this.model != null) {
            this.model.removeListDataListener(this);
        }
            
        this.model = model;
        // register with new model
        if (model != null) {
            model.addListDataListener(this);
        }
        setComponentValues();
    }


    private void setComponentValues() {
        layoutPanel.removeAll();

        // remove all buttons from the componentVec
        int numComp = componentVec.size();
        for (int i = 0; i < numComp; i++) {
            JButton btn = (JButton)componentVec.elementAt(i);
            componentVec.removeElementAt(i);
            btn.removeActionListener(this);
        }

        if (model != null) {
            int numConstraints = model.getSize();
            for (int i = 0; i < numConstraints; i++) {
                Constraints c = model.getConstraintsAt(i);
                JButton btn = new JButton(c.getName());
                btn.addActionListener(this);
                if (!c.getUseDefaultPrefSize()) {
                    btn.setPreferredSize(c.getPrefSize());
                }
                componentVec.addElement(btn);
                GridBagConstraints gbc = c.getGridBagConstraints();
                layoutPanel.add(btn, gbc);
            }
        }
    }

    public void contentsChanged(ListDataEvent evt) {
        int index0 = evt.getIndex0(); 
        int index1 = evt.getIndex1();

        for (int i = index0; i <= index1; i++) {
            JButton comp = (JButton)componentVec.elementAt(i);
            Constraints c = model.getConstraintsAt(i);
            GridBagConstraints gbc = c.getGridBagConstraints();

            layout.setConstraints(comp, gbc);
            comp.setText(c.getName());
            if (!c.getUseDefaultPrefSize()) {
                comp.setPreferredSize(c.getPrefSize());
            } else {
                comp.setPreferredSize(null);
            }
            comp.invalidate();
            comp.repaint();
        }
        layoutPanel.invalidate();
        getContentPane().validate();
        repaint();
    }

    public void intervalAdded(ListDataEvent evt) {
        int index0 = evt.getIndex0(); 
        int index1 = evt.getIndex1();

        for (int i = index0; i <= index1; i++) {
            Constraints c = model.getConstraintsAt(i);
            JButton btn = new JButton(c.getName());
            if (!c.getUseDefaultPrefSize()) {
                btn.setPreferredSize(c.getPrefSize());
            }
            btn.addActionListener(this);
            componentVec.insertElementAt(btn, i);
            GridBagConstraints gbc = c.getGridBagConstraints();
            layoutPanel.add(btn, gbc, i);
        }
        layoutPanel.invalidate();
        getContentPane().validate();
        repaint();
    }

    public void intervalRemoved(ListDataEvent evt) {
        int index0 = evt.getIndex0(); 
        int index1 = evt.getIndex1();
        for (int i = index1; i >= index0; i--) {
            layoutPanel.remove(i);
            JButton btn = (JButton)componentVec.elementAt(i);
            componentVec.removeElementAt(i);
            btn.removeActionListener(this);
        }

        layoutPanel.invalidate();
        getContentPane().validate();
        repaint();
    }

    public void actionPerformed(ActionEvent evt) {
        Object src = evt.getSource();
        if (src == packBtn) {
            pack();
        } else if (src == visibleGridBtn) {
            gridVisible = visibleGridBtn.isSelected();
            glassPane.repaint();
        } else {
            // tbd: update the selection in the list
            glassPane.repaint();
        }
    }

    class GlassPane extends JComponent {
        public void paint(Graphics g) {
            // this extra check is neccessary because the array of layout
            // dimensions is not reset whenever all components are removed
            // from GridBagLayout
            if (!gridVisible || layoutPanel.getComponentCount() == 0) {
                return;
            }


            Dimension size = getSize();
            int[][] layoutDimensions = layout.getLayoutDimensions();
            Point origin = layout.getLayoutOrigin();

            if (origin.x > 0) {
                g.setColor(Color.blue);
                g.drawLine(origin.x, 0, origin.x, size.height-1);
            }
            if (origin.y > 0) {
                g.setColor(Color.blue);
                g.drawLine(0, origin.y, size.width-1, origin.y);
            }
            
            g.setColor(Color.red);

            int numWidths = layoutDimensions[0].length;
            int curX = origin.x;
            for (int i = 0; i < numWidths; i++) {
                curX += layoutDimensions[0][i];
                g.drawLine(curX,0, curX,size.height-1);
            }

            int numHeights = layoutDimensions[1].length;
            int curY = origin.y;
            for (int i = 0; i < numHeights; i++) {
                curY += layoutDimensions[1][i];
                g.drawLine(0,curY, size.width-1,curY);
            }
        }
    }
}

